RicoMiniMap = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0")
RicoMiniMap.hasIcon = true
RicoMiniMap.hasNoText = true
RicoMiniMap.defaultPosition = "RIGHT"
RicoMiniMap.cannotDetachTooltip = true
RicoMiniMap:RegisterDB("RicoMiniMapDB")
RicoMiniMap:RegisterDefaults("profile", {
  ShowCoordinates = true,
  ShowTenths = true,
  ShowZoneText = false,
  ShowZoomButtons = false,
  Scale = 1.25,
  TopPoint = 6,
  RightPoint = 0,
})
local dewdrop = AceLibrary("Dewdrop-2.0")
local TopOffset = 21
local RightOffset = 16

RicoMiniMap_OptionTable = {
		type = "group",
		handler = RicoMiniMap,
		args = {
			RicoMiniMap_Coordinates = {
				type = "toggle",
				order = 1.0,
				name = "Show Coordinates",
				desc = "Show your location coordiates",
				get  = function() return RicoMiniMap:IsShowingCoordinates() end,
				set  = function() RicoMiniMap:ToggleShowingCoordinates() end,
			},
			RicoMiniMap_CoordinateTenths = {
				type = "toggle",
				order = 1.1,
				name = "Show Coordinate Tenths",
				desc = "Show your location with tenths",
				get  = function() return RicoMiniMap:IsShowingCoordinateTenths() end,
				set  = function() RicoMiniMap:ToggleShowingCoordinateTenths() end,
			},
			RicoMiniMap_ZoomButtons = {
				type = "toggle",
				order = 1.2,
				name = "Show Zoom Buttons",
				desc = "Show the zoom in and zoom out buttons",
				get  = function() return RicoMiniMap:IsShowingZoomButtons() end,
				set  = function() RicoMiniMap:ToggleShowingZoomButtons() end,
			},
			RicoMiniMap_ZoneLabel = {
				type = "toggle",
				order = 1.3,
				name = "Show Zone Text",
				desc = "Show the default zone text",
				get  = function() return RicoMiniMap:IsShowingZoneText() end,
				set  = function() RicoMiniMap:ToggleShowingZoneText() end,
			},
			RicoMiniMap_Scale = {
				type = "range",
				order = 1.4,
				name = "Scale",
				desc = "Scale the minimap from 50% to 150% in 25% increments",
	      max = 1.5, 
	      min = 0.5,
	      step = 0.25,
	      isPercent = true,
	      get = "GetMapScale",
	      set = "SetMapScale",
			},
			RicoMiniMap_TopPoint = {
				type = "range",
				order = 1.5,
				name = "Top Gap",
				desc = "Set the gap from the top of the screen to the top of the MiniMap",
	      max = 150, 
	      min = 0,
	      step = 2,
	      get = "GetTopPoint",
	      set = "SetTopPoint",
			},
			RicoMiniMap_RightPoint = {
				type = "range",
				order = 1.6,
				name = "Right Gap",
				desc = "Set the gap from the right side of the screen to the right side of the MiniMap",
	      max = 150, 
	      min = 0,
	      step = 2,
	      get = "GetRightPoint",
	      set = "SetRightPoint",
			},
		},
	}

function RicoMiniMap:OnInitialize()
	MinimapCluster:ClearAllPoints()
	MinimapCluster:SetScale(Minimap:GetScale() * RicoMiniMap:GetMapScale())
	MinimapCluster:SetPoint("TOPRIGHT", RightOffset - RicoMiniMap:GetRightPoint(), TopOffset - RicoMiniMap:GetTopPoint())
	--MinimapCluster:SetPoint("TOPRIGHT", right, top)
	MinimapZoomIn:ClearAllPoints()
	MinimapZoomIn:SetPoint("TOPRIGHT", -12, -133)
	MinimapZoomOut:ClearAllPoints()
	MinimapZoomOut:SetPoint("TOPRIGHT", -40, -133)
	MiniMapMailFrame:ClearAllPoints()
	MiniMapMailFrame:SetPoint("TOPRIGHT", -90, -122)
	MiniMapTrackingFrame:SetScale(Minimap:GetScale()*0.85)
	MinimapZoneTextButton:ClearAllPoints()
	MinimapZoneTextButton:SetFrameLevel(0)
	MinimapZoneTextButton:SetFrameStrata("MEDIUM")
	MinimapZoneTextButton:SetPoint("TOPRIGHT", -20, -25)
  MinimapBorder:SetTexture("Interface\\AddOns\\RicoMiniMap\\MinimapBorder")
  Minimap:SetMaskTexture("Interface\\AddOns\\RicoMiniMap\\MinimapMask")

	MinimapBorderTop:Hide()
	MinimapToggleButton:Hide()
	MiniMapWorldMapButton:Hide()
	GameTimeFrame:Hide()

	RicoMiniMap:UpdateZoomButtons()
	self.OnMenuRequest = RicoMiniMap_OptionTable
end

function RicoMiniMap_OnLoad()
	QuestWatchFrame:ClearAllPoints()
	QuestWatchFrame:SetPoint("RIGHT", "UIParent", "RIGHT", -20, 100)
	QuestWatchFrame.SetPoint = function() end
	QuestTimerFrame:ClearAllPoints()
	QuestTimerFrame:SetPoint("RIGHT", "UIParent", "RIGHT", -60, 200)
	QuestTimerFrame.SetPoint = function() end
end


function RicoMiniMap:IsShowingCoordinates()
	return self.db.profile["ShowCoordinates"]
end

function RicoMiniMap:ToggleShowingCoordinates()
	self.db.profile["ShowCoordinates"] = not self.db.profile["ShowCoordinates"]
	RicoMiniMap:Update()
	return self.db.profile["ShowCoordinates"]
end

function RicoMiniMap:IsShowingCoordinateTenths()
	return self.db.profile["ShowTenths"]
end

function RicoMiniMap:ToggleShowingCoordinateTenths()
	self.db.profile["ShowTenths"] = not self.db.profile["ShowTenths"]
	RicoMiniMap:Update()
	return self.db.profile["ShowTenths"]
end

function RicoMiniMap:IsShowingZoomButtons()
	return self.db.profile["ShowZoomButtons"]
end

function RicoMiniMap:ToggleShowingZoomButtons()
	self.db.profile["ShowZoomButtons"] = not self.db.profile["ShowZoomButtons"]
	RicoMiniMap:UpdateZoomButtons()
	return self.db.profile["ShowZoomButtons"]
end

function RicoMiniMap:UpdateZoomButtons()
	if RicoMiniMap:IsShowingZoomButtons() then
		MinimapZoomIn:Show()
  	MinimapZoomOut:Show()	
	else
		MinimapZoomIn:Hide()
  	MinimapZoomOut:Hide()	
	end
end

function RicoMiniMap:IsShowingZoneText()
	return self.db.profile["ShowZoneText"]
end

function RicoMiniMap:ToggleShowingZoneText()
	self.db.profile["ShowZoneText"] = not self.db.profile["ShowZoneText"]
	if RicoMiniMap:IsShowingZoneText() then
		MinimapZoneTextButton:Show()
	else
		MinimapZoneTextButton:Hide()
	end
	return self.db.profile["ShowZoneText"]
end

function RicoMiniMap:GetMapScale()
	return self.db.profile["Scale"]
end

function RicoMiniMap:SetMapScale(value)
	MinimapCluster:SetScale(Minimap:GetScale() * value)
	self.db.profile["Scale"] = value
end

function RicoMiniMap:GetTopPoint()
	return self.db.profile["TopPoint"]
end

function RicoMiniMap:SetTopPoint(value)
	self.db.profile["TopPoint"] = value
	MinimapCluster:ClearAllPoints()
	MinimapCluster:SetPoint("TOPRIGHT", RightOffset - RicoMiniMap:GetRightPoint(), TopOffset - value)
end

function RicoMiniMap:GetRightPoint()
	return self.db.profile["RightPoint"]
end

function RicoMiniMap:SetRightPoint(value)
	self.db.profile["RightPoint"] = value
	MinimapCluster:ClearAllPoints()
	MinimapCluster:SetPoint("TOPRIGHT", RightOffset - value, TopOffset - RicoMiniMap:GetTopPoint())
end


------------------
-- Coordinate text
function RicoMiniMap_updateLocText()
	if RicoMiniMap:IsShowingCoordinates() then
		local x, y = GetPlayerMapPosition('player')
		if x == 0 and y == 0 then 
			RicoMinimap_LocText:SetText('')
		else
			if RicoMiniMap:IsShowingCoordinateTenths() then
				RicoMinimap_LocText:SetText(string.format('%.1f, %.1f', x*100, y*100))
			else
				RicoMinimap_LocText:SetText(string.format('%.0f, %.0f', x*100, y*100))
			end
		end
	else
		RicoMinimap_LocText:SetText('')
	end
	MinimapCluster:ClearAllPoints()
	MinimapCluster:SetPoint("TOPRIGHT", RightOffset - RicoMiniMap:GetRightPoint(), TopOffset - RicoMiniMap:GetTopPoint())
end

-------------------------------
-- Zooming with the mouse wheel
function MiniMap_OnMouseWheel(value)
		if value > 0 then
			RicoMinimap_ZoomIn()
		elseif value < 0 then
			RicoMinimap_ZoomOut()
		end
end

function RicoMinimap_ZoomIn()
	if Minimap:GetZoom() == 5 then
		Minimap:SetZoom(5)
	else
		Minimap:SetZoom(Minimap:GetZoom() + 1)
	end
end

function RicoMinimap_ZoomOut()
	if Minimap:GetZoom() == 0 then
		Minimap:SetZoom(0)
	else
		Minimap:SetZoom(Minimap:GetZoom() - 1)
	end
end
